/*
 *	Title:		Escavalon - Interactive Fiction System
 *	Version:	Beta
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of Escavalon
 *	******************************************
 *	
 *	Escavalon is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */



#if !defined(FANEL_AVALON_CLASSES_H__INCLUDED_)
#define FANEL_AVALON_CLASSES_H__INCLUDED_

#include <string>
#include <list>

#include "fnmaps.h"

using namespace std;



enum ESCAVALON_BASE_CONSTANTS
{
	AV_INVALID = 0,		// wrong direction, etc
	AV_INITIAL = 1,		// initial state for AvObject e AvLocation
	AV_INVENTORY = -2,		// move an object to inventory
	AV_LIMBO = -3,			// move an object to limbo
	AV_ANY_PLACE = -4,		// action performed everywhere
	AV_ANY_TERM = -5,		// action performed with any word

	AV_BASE_ALLOWED = 100
};


/*inline bool IsValidID(int nID)
{
	return nID > 0;
}
inline bool IsValidCustomVerb(int nID)
{
	return (nID > 0 && nID < AV_BASE_ALLOWED);
}
inline bool IsValidCustomState(int nID)
{
	return (nID > AV_INITIAL);
}
inline bool IsValidLocation(int nID)
{
	return (IsValidID(nID) || nID == AV_INVALID);
}
inline bool IsValidObjectLocation(int nID)
{
	return (IsValidID(nID) || nID == AV_INVENTORY || nID == AV_LIMBO);
}*/


//class AvArchive; // TODO definire!
class AvalonGame;
class AvDesigner;

///////////////////////////////////////////////////////////////

// non puo' essere istanziata

class AvEntity  
{
public:
	virtual ~AvEntity() { }
	int GetID() const { return _nID; }
protected:
	AvEntity(int nID);
private:
	int _nID;
};

///////////////////////////////////////////////////////////////

// questa classe deve essere istanziata solo da AvDesigner

class AvTerm : public AvEntity
{
public:
	~AvTerm() { }
	const list<string>& GetWordList() const { return _words; }
protected:
	AvTerm(int nID) : AvEntity(nID) { }
	AvTerm(int nID, const string& sWords);
	void AddWords(const string& sWords);
private:
	list<string> _words;
	friend class AvDesigner;
};

///////////////////////////////////////////////////////////////


class AvState : public AvEntity
{
public:
	~AvState() { }
	inline AvState(int nID, const string& sShortDesc, const string& sLongDesc);
	virtual const string& GetShortDesc() const { return _sShortDesc; }
	virtual const string& GetLongDesc() const  { return _sLongDesc; }
private:
	string _sShortDesc;
	string _sLongDesc;
};

AvState::AvState(int nID, const string& sShortDesc, const string& sLongDesc) 
: AvEntity(nID), _sShortDesc(sShortDesc), _sLongDesc(sLongDesc)
 { }


///////////////////////////////////////////////////////////////

// non puo' essere istanziata

class AvEntityWithState : public AvEntity
{
public:
	~AvEntityWithState() { }
	const AvState& GetState() const;
	int	GetStateID() const { return _nCurrentStateID; }
	virtual void SetState(int nStateID);
	const string& GetShortDesc() const;
	const string& GetLongDesc() const;

protected:
	AvEntityWithState(int nID, const string& sShortDesc, const string& sLongDesc);
	void AddState(const AvState& state);
private:
	Fanel::IntegerMap<AvState> _states;
	int _nCurrentStateID;

	// TODO metodo pubblico Save(AvArchive);
	//friend class AvDesigner; //TODO serve?!?
};

///////////////////////////////////////////////////////////////

// questa classe deve essere istanziata solo da AvDesigner

class AvLocation : public AvEntityWithState
{
public:
	~AvLocation() { }
	bool IsVisited() const { return _bVisited; }
	const string& GetDesc() const;
	void SetVisited();
	virtual void SetState(int nStateID); // override

	int North() const { return _nNorth; }
	int South() const { return _nSouth; }
	int East() const { return _nEast; }
	int West() const { return _nWest; }
	int Up() const { return _nUp; }
	int Down() const { return _nDown; }

protected:
	AvLocation(int nID, const string& sShortDesc, const string& sLongDesc,
			   int north, int south, int east, int west, int up, int down);
private:
	bool _bVisited;
	int _nNorth;
	int _nSouth;
	int _nEast;
	int _nWest;
	int _nUp;
	int _nDown;

	friend class AvDesigner;
	friend ostream& operator << (ostream& out, const AvLocation& location);
	friend istream& operator >> (istream& in, AvLocation& location);
	//friend void ReadFromString(const char* buffer, AvLocation& location); 
};

ostream& operator << (ostream& out, const AvLocation& location);
istream& operator >> (istream& in, AvLocation& location);

/*void ReadFromString(const char* buffer, AvLocation& location);*/
///////////////////////////////////////////////////////////////

// questa classe deve essere istanziata solo da AvDesigner

class AvObject : public AvEntityWithState
{
public:
	enum OBJECT_TYPE { UNSPECIFIED, THING, PLANT, ANIMAL, PERSON };

	~AvObject() { }
	int GetLocationID() const { return _nLocation; }
	OBJECT_TYPE GetType() const { return _type; }
	bool IsFixed() const { return _bFixed; }

	void MoveTo(int nLocation);

protected:
	AvObject(int nID, const string& sShortDesc, const string& sLongDesc,
			int nInitialLocation, bool bFixed, OBJECT_TYPE type = UNSPECIFIED);
private:
	int _nLocation;
	bool _bFixed;	// indica se si puo' prendere l'oggetto
	OBJECT_TYPE _type;

	friend class AvDesigner;
	friend ostream& operator << (ostream& out, const AvObject& obj);
	friend istream& operator >> (istream& in, AvObject& obj);
};

ostream& operator << (ostream& out, const AvObject& obj);
istream& operator >> (istream& in, AvObject& obj);

///////////////////////////////////////////////////////////////

// classi che rappresentano le variabili custom

class AvIntVariable
{
public:
	AvIntVariable() : nVal(0) { }
	int nVal;
//	friend ostream& operator << (ostream& out, const AvIntVariable& variable);
//	friend istream& operator >> (istream& in, AvIntVariable& variable);
};

ostream& operator << (ostream& out, const AvIntVariable& variable);
istream& operator >> (istream& in, AvIntVariable& variable);

class AvStrVariable
{
public:
	string sVal;
//	friend ostream& operator << (ostream& out, const AvStrVariable& variable);
//	friend istream& operator >> (istream& in, AvStrVariable& variable);
};

ostream& operator << (ostream& out, const AvStrVariable& variable);
istream& operator >> (istream& in, AvStrVariable& variable);

///////////////////////////////////////////////////////////////

// rappresentazione dei timer
// nota: un valore del timer negativo o nullo indica timer non attivo

class AvTimer : public AvEntity
{
public:
	~AvTimer() { }
	int GetTime() const { return _nTime; }
	void SetTime(int nTime);
	bool IsActive() const { return _nTime > 0; }
	int Tick();

protected:
	AvTimer(int nID);
private:
	int _nTime;
	friend class AvDesigner;
};

ostream& operator << (ostream& out, const AvTimer& variable);
istream& operator >> (istream& in, AvTimer& variable);



#endif

