/*
 *	Title:		Escavalon - Interactive Fiction System
 *	Version:	Beta
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of Escavalon
 *	******************************************
 *	
 *	Escavalon is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */



// suppress "truncated debug info" warnings (VC++ specific)
#ifdef _MSC_VER
#pragma warning (disable: 4786)
#pragma warning (disable: 4788)
#endif // _MSC_VER


#include "fnconsol.h"
#include "fnutils.h"

#include <iostream>
#include <time.h>

using namespace std;

void ConsoleIOManager::Read(string& str)
{
	do
	{
		cout << " > ";
		Fanel::ReadLine(cin, str);
	}	while(str.empty());
}

void ConsoleIOManager::Read(int& i)
{
	while( (cout << " > ") && !(cin >> i))
	{
		Fanel::ClearInput(cin);
	}
	Fanel::WaitForEnter();
}

void ConsoleIOManager::Read(double& d)
{
	while( (cout << " > ") && !(cin >> d))
	{
		Fanel::ClearInput(cin);
	}
	Fanel::WaitForEnter();
}

void ConsoleIOManager::ReadTokenized(vector<string>& lst)
{
	string str, strEp;
	do
	{
		Read(str);
//		Fanel::Str::Normalize(str);
		Fanel::Str::Epurate(str, strEp);
		strEp = Fanel::Str::Trim(strEp);
	}	while(strEp.empty());

	Fanel::Str::ToUpper(strEp);
	Fanel::Str::Tokenize(strEp, lst);
}

bool ConsoleIOManager::ReadYesNo(const string& strMessage)
{
	string str;
	do
	{
		cout << strMessage;
		Fanel::ReadLine(cin, str);
		Fanel::Str::ToUpper(str);

	}	while (str[0] != 'S' && str[0] != 'N');

	return (str[0] == 'S');
}

void ConsoleIOManager::ReadFileName(string& str, bool bLoad)
{
	if(bLoad)
	{
		cout << "Nome della partita da caricare:" << endl;
	}
	else
	{
		cout << "Dai un nome alla partita da salvare:" << endl;
	}
	Read(str);
}

void ConsoleIOManager::Write(const string& str)
{
	cout << str.c_str();
}

void ConsoleIOManager::Write(const char* str)
{
	cout << str;
}

void ConsoleIOManager::Write(int i)
{
	cout << i;
}

void ConsoleIOManager::Write(double d)
{
	cout << d;
}

void ConsoleIOManager::WriteLine()
{
	cout << endl;
}


void ConsoleIOManager::Wait(int nSecs)
{
	int i = 0;
    time_t t0;
    time(&t0);    // get current time
    while (difftime(time(0), t0) < nSecs)
    {
		i++; // do something to prevent loop optimization
    }
}

void ConsoleIOManager::WaitForEnter()
{
	cout << "Premi <INVIO> per continuare." << endl;
	Fanel::WaitForEnter();
}





