/*
 *	Title:		Escavalon - Interactive Fiction System
 *	Version:	Beta
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of Escavalon
 *	******************************************
 *	
 *	Escavalon is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */


// suppress "truncated debug info" warnings (VC++ specific)
#ifdef _MSC_VER
#pragma warning (disable: 4786)
#pragma warning (disable: 4788)
#endif // _MSC_VER


#include "fnutils.h"

#include <iostream>
//#include <limits>
//#include <cctype>

using namespace std;


#define FNUTILS_MAX_BUFFER_SIZE 512
#define FNUTILS_MAX_CHAR_TO_IGNORE 1024

namespace Fanel
{

/////////////////////////////////////
// input/output  
/////////////////////////////////////

void WaitForEnter()
{
	cin.ignore(FNUTILS_MAX_CHAR_TO_IGNORE, '\n');
}

void WaitForEnter(std::istream& input)
{
	input.ignore(FNUTILS_MAX_CHAR_TO_IGNORE, '\n');
}

void ClearInput(istream& is)
{
	is.clear();
	is.ignore(FNUTILS_MAX_CHAR_TO_IGNORE, '\n');
/*	char D;
	while (is.rdbuf()->in_avail())
		 is.rdbuf()->sgetn(&D, 1); 
	return ios::goodbit == is.rdstate();*/
}

void ReadLine(istream& input, string& str)
{
	char buffer[FNUTILS_MAX_BUFFER_SIZE];
	input.getline(buffer, FNUTILS_MAX_BUFFER_SIZE-1);
	str = buffer;
}

/////////////////////////////////////
// string operations
/////////////////////////////////////

namespace Str
{
	void ToUpper(string& str)
	{
		for ( string::iterator i = str.begin(); i != str.end(); ++i )
			*i = toupper( *i );
	}

	void ToUpper(const string& str, string& strOut)
	{
		strOut.resize(str.length());
		string::const_iterator i	= str.begin();
		string::iterator out		= strOut.begin();
		for ( ; i != str.end(); ++i, ++out )
			*out = toupper( *i );
	}

	void Tokenize(const char* str, list<string>& lst)
	{
		lst.clear();
		char buffer[FNUTILS_MAX_BUFFER_SIZE];
		istrstream stream(str);
		while(stream >> buffer)
		{
			lst.push_back(string(buffer));
		}
	}

	void Tokenize(const string& str, list<string>& lst)
	{
		Tokenize(str.c_str(), lst);
	}

	void Tokenize(const char* str, vector<string>& lst)
	{
		lst.clear();
		char buffer[FNUTILS_MAX_BUFFER_SIZE];
		istrstream stream(str);
		while(stream >> buffer)
		{
			lst.push_back(string(buffer));
		}
	}

	void Tokenize(const string& str, vector<string>& lst)
	{
		Tokenize(str.c_str(), lst);
	}

	void Epurate(const string& in, string& out)
	{
		string::const_iterator iterIn = in.begin();
		out.erase(out.begin(), out.end());
		out.reserve(in.size());
		for(;iterIn!=in.end(); ++iterIn)
		{
			if (isalpha(*iterIn) || isspace(*iterIn) || isdigit(*iterIn))
				out += *iterIn;
			else
				out += " ";
		}
	}

	string TrimRight( const string& s ) {
	  if (s.empty())
		  return s;
	  string::size_type pos = s.find_last_not_of(" \t\n");
	  if (pos > s.length())
		  return string();
	  return s.substr( 0, pos+1 );
	}

	string TrimLeft( const string& s ) {
	  if (s.empty())
		  return s;
	  string::size_type pos = s.find_first_not_of(" \t\n");
	  if (pos > s.length())
		  return string();
	  return s.substr( pos, s.size()-pos );
	}

	string Trim( const string& s ) {
	  if (s.empty())
		  return s;
	  string::size_type lpos = s.find_first_not_of(" \t\n");
	  string::size_type rpos = s.find_last_not_of(" \t\n");
	  if (lpos > s.length() || rpos > s.length())
		  return string();
	  return s.substr( lpos, rpos-lpos+1 );
	}

	// char conversion functions

/*	static const char* const DEFAULT_MAPPINGS [] = 
	{	"A", "",
		"E", "",
		"I", "",
		"O", "",
		"U", "",
		"Y", "",
		"C", "",
		"N", "",
		"", ""
	};

	char Normalize(char c)
	{
		return Normalize(c, DEFAULT_MAPPINGS);
	}

	void Normalize(const string& strIn, string& strOut)
	{
		Normalize(strIn, strOut, DEFAULT_MAPPINGS);
	}

	void Normalize(string& str)
	{
		Normalize(str, DEFAULT_MAPPINGS);
	}

	char Normalize(char c, const char* const mappings[])
	{
		string strCurrent, strChars;
		strCurrent = mappings[0];
		strChars = mappings[1];
		int i = 0, found;
		while(!strCurrent.empty())
		{
			found = strChars.find(c);
			if (found >= 0 && found < strChars.length())
				return strCurrent[0];
			i+=2;
			strCurrent = mappings[i];
			strChars = mappings[i+1];
		}
		return c;
	}

	void Normalize(const string& strIn, string& strOut, const char* const mappings[])
	{
		strOut.erase(strOut.begin(), strOut.end());
		strOut.reserve(strIn.length());
		for(string::const_iterator it = strIn.begin(); it!=strIn.end(); ++it)
		{
			strOut += Normalize(*it, mappings);
		}
	}

	void Normalize(string& strIn, const char* const mappings[])
	{
		for(string::iterator it = strIn.begin(); it!=strIn.end(); ++it)
		{
			*it = Normalize(*it, mappings);
		}
	}
*/

} // end namespace Str



} // end namespace Fanel

